$tempPath = $env:TEMP 
function WriteMessage([string[]]$message){
	if($message -isnot [system.array]) { $message = @($message)}
	$text = -join $message

	$text | out-file "$tempPath\deinstallSR.txt" -Append
	write-host $text
}
function RemoveIisAppPool($poolName){
	$sysPath = [Environment]::GetFolderPath("System")
	WriteMessage "try to remove ", $poolName
	$cmd = 'appcmd.exe delete apppool /apppool.name:"' + $poolName + '"'
	& "$sysPath\CMD.EXE" /c $cmd
}


WriteMessage "SiteRemote uninstall script."


# am I running in 32 bit shell?
if ($pshome -like "*syswow64*") {
    WriteMessage "Restarting script under 64 bit powershell"
    write-warning "Restarting script under 64 bit powershell"
 
	if ($psscriptroot  -eq $null) { $psscriptroot = Split-Path $($MyInvocation.InvocationName) -Parent } # Version 2 Power shell
	
    # relaunch this script under 64 bit shell
    # if you want powershell 2.0, add -version 2 *before* -file parameter
    & (join-path ($pshome -replace "syswow64", "sysnative") powershell.exe) -ExecutionPolicy Remotesigned -file `
        (join-path $psscriptroot $myinvocation.mycommand)

	WriteMessage "Restarted x64 instance"
	write-warning "Restarted x64 instance"
    # exit 32 bit script
    exit
}

# start of script for 64 bit powershell

# found uninstall information
$sysPath = [Environment]::GetFolderPath("System")
$appCmd = "appcmd.exe"
$pFixed = "HKLM:SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\"
$keys = Get-ChildItem -Path $pFixed -Recurse | Where-Object -FilterScript {($_.GetValue("DisplayName", $null) -eq "SiteRemote Server")}
$uninstallId = $null
$installPath = $null
foreach($k in $keys) {
	$uninstallId = $k.Name.SubString($k.Name.LastIndexOf("\") + 1)
	$installPath = (Get-ItemProperty -Path "$pFixed$uninstallId").InstallLocation
}

#remove Users
$Computername = $env:COMPUTERNAME
$ADSIComp = [adsi]"WinNT://$Computername"
try
{
	$ADSIComp.Delete('User','SITEREMOTE') 
}
catch 
{
	WriteMessage "Failed to remove the user SITEREMOTE"
}
try
{
	$ADSIComp.Delete('User','SITEREMOTEADMIN') 
}
catch 
{
	WriteMessage "Failed to remove the user SITEREMOTEADMIN"
}
try
{
	$ADSIComp.Delete('User','SITEREMOTECMS') 
}
catch 
{
	WriteMessage "Failed to remove the user SITEREMOTECMS"
}
WriteMessage "removed users"

#remove scheduled task
SchTasks /Delete /TN "\SiteCaster CMS backup" /f
WriteMessage "removed scheduled backup task"

#remove Mongo service
net stop "SiteCasterMongoDB"
sc.exe delete "SiteCasterMongoDB"
WriteMessage "removed mongo db"


if (($uninstallId -eq $null) -or ($installPath -eq $null))
{
	WriteMessage "Installation not found"
	exit
}
WriteMessage "Installation found: ", $installPath
cd $sysPath"\inetsrv\"

#remove pools
RemoveIisAppPool('SiteRemote')
RemoveIisAppPool('SiteRemoteAdmin')
RemoveIisAppPool('SiteRemote WEBDAV')
RemoveIisAppPool('SiteRemoteWebDAV')
RemoveIisAppPool('SiteRemoteCms')

WriteMessage "Processing IIS..."
for ($iisNumber=1; $iisNumber -le 10; $iisNumber++){
	
	$pFixed = $installPath + "\InstallBackup\site_" + $iisNumber + ".xml"
	if (Test-Path $pFixed)
	{
		WriteMessage "Read Web backup from ", $pFixed 

		#remove site
		WriteMessage "removing SiteRemote site"
		cd $sysPath"\inetsrv\"
		$cmd = $appCmd + ' list site /id:' + $iisNumber + ' /xml | ' + $appCmd + ' delete site /in'
		& "$sysPath\CMD.EXE" /c $cmd
		WriteMessage "removed SiteRemote site"

		#restore backup
		$cmd = 'type "'+ $pFixed+ '" |  ' + $appCmd + ' add site /in'
		& "$sysPath\CMD.EXE" /c $cmd
		WriteMessage "restored site from the backup"
		break		
	}
}

# uninstall services
WriteMessage "Uninstalling services..."

net stop "SiteRemote Server"

$servicePath = $installPath + "\service"
$ejabberdPath = $installPath + "\ejabberd\bin"
if ([System.IO.Directory]::Exists($ejabberdPath))
{
	cd "$ejabberdPath"
	.\erlsrv remove SiteRemote_XMPP_Server
	.\epmd -kill
	.\sleep 1
	taskkill /F /IM Provisio.Longinus.Xmpp.Ejabberd.Authentication.exe

	# remove ODBC source from registry
	WriteMessage "Removing Ejabberd ODBC source from registry"
	Powershell.exe  -executionpolicy remotesigned -File "remove_odbc.ps1"
}

cd "$servicePath"
.\SiteRemoteServerService.exe /uninstall
cd ..



# clean registry
WriteMessage "Clean registry..."

$idKeys = Get-ChildItem -Path HKLM:Software\Classes\Installer\Features -Recurse | Where-Object -FilterScript {($_.SubKeyCount -le 1) -and ($_.ValueCount -eq 3) -and ($_.GetValue("SiteRemote", $null) -ne $null)}

#features can be empty
if ($idKeys -eq $null) {
	$idKeys = Get-ChildItem -Path HKLM:Software\Classes\Installer\Products -Recurse | Where-Object -FilterScript { ($_.GetValue("ProductName", $null) -eq "SiteRemote Server")}
}

if ($idKeys -ne $null)
{
	if($idKeys -isnot [system.array]) { $idKeys = @($idKeys)}
	if ($idKeys.length -eq 1)
	{
		$id = $idKeys[0].Name
		$id = $id.SubString($id.LastIndexOf("\") + 1)
		
		$keys = Get-ChildItem -Path HKLM:Software\Classes\Installer -Recurse | Where-Object -FilterScript {($_.Name.EndsWith($id))}
		foreach($k in $keys) {
			$pFixed = "HKLM:" + $k.Name.SubString($k.Name.IndexOf("\") + 1)
			Remove-Item $pFixed -recurse
			WriteMessage "removed", $pFixed
		} 
		
		$keys= Get-ChildItem -Path HKLM:Software\Classes\Installer -Recurse | Where-Object -FilterScript {($_.GetValue($id, $null) -ne $null)}
		foreach($k in $keys) {
			$pFixed = "HKLM:" + $k.Name.SubString($k.Name.IndexOf("\") + 1)
			Remove-Item $pFixed -recurse
			# WriteMessage "removed" $k.Name.SubString($k.Name.IndexOf("\") + 1)
		} 
		
		# ends with or contains one
		$keys = Get-ChildItem -Path HKLM:SOFTWARE\Microsoft\Windows\CurrentVersion\Installer -Recurse | Where-Object -FilterScript {(($_.GetValue($id, $null) -ne $null) -and ($_.ValueCount -eq 1)) -or ($_.Name.EndsWith($id))}
		foreach($k in $keys) {
			$pFixed = "HKLM:" + $k.Name.SubString($k.Name.IndexOf("\") + 1)
			Remove-Item $pFixed -recurse
			# WriteMessage "removed", $pFixed
			$i++;
		} 
		WriteMessage "removed ", $i, " entries from HKLM:SOFTWARE\Microsoft\Windows\CurrentVersion\Installer"
	}
	if ($idKeys.length -ne 1)
	{
		WriteMessage "Error: could not find registry keys"
	}
}

#folder list
$pFixed = "HKLM:SOFTWARE\Microsoft\Windows\CurrentVersion\Installer\Folders"
$properties = Get-ItemProperty -Path $pFixed
$i = 0;
if ($properties -ne $null)
{
	if($propertyName -isnot [system.array]) { $propertyName = @($propertyName)}
	
	foreach($propertyName in $properties | Get-Member) {
		if ($propertyName.Name.StartsWith($installPath))
		{
			Remove-ItemProperty -Path $pFixed -Name $propertyName.Name
			# WriteMessage "removed " $propertyName.Name
			$i++;
		}
	} 
}
WriteMessage "removed ", $i, " entries from ", $pFixed

#shared dll
$pFixed = "HKLM:SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\SharedDlls"
$properties = Get-ItemProperty -Path $pFixed
$i = 0;
if ($properties -ne $null)
{
	if($propertyName -isnot [system.array]) { $propertyName = @($propertyName)}
	
	foreach($propertyName in $properties | Get-Member) {
		if ($propertyName.Name.StartsWith($installPath))
		{
			Remove-ItemProperty -Path $pFixed -Name $propertyName.Name
			# WriteMessage "removed " $propertyName.Name
			$i++;
		}
	}
}	
WriteMessage "removed ", $i ", entries from ", $pFixed

#uninstall
# remove HKLM:SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\<id>
$keys = Get-ChildItem -Path HKLM:SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall -Recurse | Where-Object -FilterScript {($_.GetValue("DisplayName", $null) -eq "SiteRemote Server")}
if ($keys -ne $null)
{
	if($keys -isnot [system.array]) { $keys = @($keys)}
	
	foreach($k in $keys) {
		$pFixed = "HKLM:" + $k.Name.SubString($k.Name.IndexOf("\") + 1)
		Remove-Item $pFixed -recurse
		WriteMessage "removed", $pFixed
	} 
}
$pFixed = "HKLM:SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\InstallShield Uninstall Information\$uninstallId"
Remove-Item  $pFixed -recurse
WriteMessage "removed", $pFixed
	

# product settings
Remove-Item -Path "HKLM:SOFTWARE\PROVISIO" -Recurse
Remove-Item -Path "HKLM:SOFTWARE\Wow6432Node\PROVISIO" -Recurse


# delete files
#Remove-Item -Path "\*" -Recurse -Exclude "Config"  -EA SilentlyContinue # not function in Version 3.0
WriteMessage "removing files from ", $installPath, " ..."

Get-ChildItem -Path $installPath -Exclude "Config", "Common", "MongoData" | foreach ($_) {
 
		$Stoploop = $false
		[int]$Retrycount = "0"
 
		do {
			try {
				Remove-Item $_.fullname -Force -Recurse  -ErrorAction Stop
				$Stoploop = $true
				WriteMessage $_.fullname
			}
			catch {
				if ($Retrycount -gt 3){
					WriteMessage "Could not remove directory after 3 retrys. ", $_.fullname
					$Stoploop = $true
				}
				else {
					WriteMessage "Could not remove directory retrying in 30 seconds..."
					Start-Sleep -Seconds 30
					$Retrycount = $Retrycount + 1
				}
			}
		}
		While ($Stoploop -eq $false)
   }

WriteMessage "removed files from ", $installPath

# SIG # Begin signature block
# MIIXDgYJKoZIhvcNAQcCoIIW/zCCFvsCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUMVpIpubQ72J9ZPW95So2TfUP
# vkugghJeMIID7jCCA1egAwIBAgIQfpPr+3zGTlnqS5p31Ab8OzANBgkqhkiG9w0B
# AQUFADCBizELMAkGA1UEBhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTEUMBIG
# A1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoTBlRoYXd0ZTEdMBsGA1UECxMUVGhh
# d3RlIENlcnRpZmljYXRpb24xHzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBpbmcg
# Q0EwHhcNMTIxMjIxMDAwMDAwWhcNMjAxMjMwMjM1OTU5WjBeMQswCQYDVQQGEwJV
# UzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xMDAuBgNVBAMTJ1N5bWFu
# dGVjIFRpbWUgU3RhbXBpbmcgU2VydmljZXMgQ0EgLSBHMjCCASIwDQYJKoZIhvcN
# AQEBBQADggEPADCCAQoCggEBALGss0lUS5ccEgrYJXmRIlcqb9y4JsRDc2vCvy5Q
# WvsUwnaOQwElQ7Sh4kX06Ld7w3TMIte0lAAC903tv7S3RCRrzV9FO9FEzkMScxeC
# i2m0K8uZHqxyGyZNcR+xMd37UWECU6aq9UksBXhFpS+JzueZ5/6M4lc/PcaS3Er4
# ezPkeQr78HWIQZz/xQNRmarXbJ+TaYdlKYOFwmAUxMjJOxTawIHwHw103pIiq8r3
# +3R8J+b3Sht/p8OeLa6K6qbmqicWfWH3mHERvOJQoUvlXfrlDqcsn6plINPYlujI
# fKVOSET/GeJEB5IL12iEgF1qeGRFzWBGflTBE3zFefHJwXECAwEAAaOB+jCB9zAd
# BgNVHQ4EFgQUX5r1blzMzHSa1N197z/b7EyALt0wMgYIKwYBBQUHAQEEJjAkMCIG
# CCsGAQUFBzABhhZodHRwOi8vb2NzcC50aGF3dGUuY29tMBIGA1UdEwEB/wQIMAYB
# Af8CAQAwPwYDVR0fBDgwNjA0oDKgMIYuaHR0cDovL2NybC50aGF3dGUuY29tL1Ro
# YXd0ZVRpbWVzdGFtcGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAOBgNV
# HQ8BAf8EBAMCAQYwKAYDVR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0y
# MDQ4LTEwDQYJKoZIhvcNAQEFBQADgYEAAwmbj3nvf1kwqu9otfrjCR27T4IGXTdf
# plKfFo3qHJIJRG71betYfDDo+WmNI3MLEm9Hqa45EfgqsZuwGsOO61mWAK3ODE2y
# 0DGmCFwqevzieh1XTKhlGOl5QGIllm7HxzdqgyEIjkHq3dlXPx13SYcqFgZepjhq
# IhKjURmDfrYwggSUMIIDfKADAgECAg5IG2oHJtLoPyYC1IJazTANBgkqhkiG9w0B
# AQsFADBMMSAwHgYDVQQLExdHbG9iYWxTaWduIFJvb3QgQ0EgLSBSMzETMBEGA1UE
# ChMKR2xvYmFsU2lnbjETMBEGA1UEAxMKR2xvYmFsU2lnbjAeFw0xNjA2MTUwMDAw
# MDBaFw0yNDA2MTUwMDAwMDBaMFoxCzAJBgNVBAYTAkJFMRkwFwYDVQQKExBHbG9i
# YWxTaWduIG52LXNhMTAwLgYDVQQDEydHbG9iYWxTaWduIENvZGVTaWduaW5nIENB
# IC0gU0hBMjU2IC0gRzMwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCN
# hVUjqR9Tr8ntNsIptW7Y+UL1IYuH8EOhL8A/l+MYR9SWToirxmPptkkVhfHZm3sb
# /dhKzG1OhkAXzXu6Ryi11hRADIbuHksz8yxV7iGM2rbB/rBOOq5Rn6UU4xDmk8r6
# +V2xkIfv+DUt/KJcJu57FYsf2cOhlzVBszD9chOtkZc6znKdBgp1PB+Y48sYL4yf
# CEqRCtnZNdmDknZiXt+DruTWAU7M8zxwYVg3HxTjaqCva/TZ0mwsGTBdoG9S39Gc
# yeAN2XURZZbZQ7SnkDmuRxxUy7GVbiXejvESHPDXbucUTbMaZdaESlfuBK9iOMUQ
# m0OOUrg+tq6eLJf/jnTvAgMBAAGjggFkMIIBYDAOBgNVHQ8BAf8EBAMCAQYwHQYD
# VR0lBBYwFAYIKwYBBQUHAwMGCCsGAQUFBwMJMBIGA1UdEwEB/wQIMAYBAf8CAQAw
# HQYDVR0OBBYEFA8656yUkXQtlgJzg62cLkk/GapUMB8GA1UdIwQYMBaAFI/wS3+o
# LkUkrk1Q+mOai97i3Ru8MD4GCCsGAQUFBwEBBDIwMDAuBggrBgEFBQcwAYYiaHR0
# cDovL29jc3AyLmdsb2JhbHNpZ24uY29tL3Jvb3RyMzA2BgNVHR8ELzAtMCugKaAn
# hiVodHRwOi8vY3JsLmdsb2JhbHNpZ24uY29tL3Jvb3QtcjMuY3JsMGMGA1UdIARc
# MFowCwYJKwYBBAGgMgEyMAgGBmeBDAEEATBBBgkrBgEEAaAyAV8wNDAyBggrBgEF
# BQcCARYmaHR0cHM6Ly93d3cuZ2xvYmFsc2lnbi5jb20vcmVwb3NpdG9yeS8wDQYJ
# KoZIhvcNAQELBQADggEBABWEKAztocMZgttjJ0HXzGN91rzPNpvP0l1MAotYGhYI
# erGYmX/YM4pcnmIKmuqQwsVjBAvoh1gGAAeCWcOolDLZ4BRNoNUj4MfduvBp4kpF
# ZS1NSZB4ZjIOsGjAsIiwju1cBvhcEEg/I3O6O1OEUoDN8LMVyBEKiwV4RlkI1L63
# /0v1nGpMnHaiEYVFjNQ37lDd4TM0qaEfOgvxVkSKb7Mz0LGO0QxgB+4ywvAkb7+v
# +4EBdmfEo+jgq9wzVSjjZ0c862qk35Tp9KbAgdFSmFGm1gK3POpK79C6ZdI3g1NL
# fmd8jED2BxywrwQG3PhsRohynOtOncOwuVSjuU6XyhQwggSjMIIDi6ADAgECAhAO
# z/Q4yP6/NW4E2GqYGxpQMA0GCSqGSIb3DQEBBQUAMF4xCzAJBgNVBAYTAlVTMR0w
# GwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjEwMC4GA1UEAxMnU3ltYW50ZWMg
# VGltZSBTdGFtcGluZyBTZXJ2aWNlcyBDQSAtIEcyMB4XDTEyMTAxODAwMDAwMFoX
# DTIwMTIyOTIzNTk1OVowYjELMAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFudGVj
# IENvcnBvcmF0aW9uMTQwMgYDVQQDEytTeW1hbnRlYyBUaW1lIFN0YW1waW5nIFNl
# cnZpY2VzIFNpZ25lciAtIEc0MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEAomMLOUS4uyOnREm7Dv+h8GEKU5OwmNutLA9KxW7/hjxTVQ8VzgQ/K/2plpbZ
# vmF5C1vJTIZ25eBDSyKV7sIrQ8Gf2Gi0jkBP7oU4uRHFI/JkWPAVMm9OV6GuiKQC
# 1yoezUvh3WPVF4kyW7BemVqonShQDhfultthO0VRHc8SVguSR/yrrvZmPUescHLn
# kudfzRC5xINklBm9JYDh6NIipdC6Anqhd5NbZcPuF3S8QYYq3AhMjJKMkS2ed0Qf
# aNaodHfbDlsyi1aLM73ZY8hJnTrFxeozC9Lxoxv0i77Zs1eLO94Ep3oisiSuLsdw
# xb5OgyYI+wu9qU+ZCOEQKHKqzQIDAQABo4IBVzCCAVMwDAYDVR0TAQH/BAIwADAW
# BgNVHSUBAf8EDDAKBggrBgEFBQcDCDAOBgNVHQ8BAf8EBAMCB4AwcwYIKwYBBQUH
# AQEEZzBlMCoGCCsGAQUFBzABhh5odHRwOi8vdHMtb2NzcC53cy5zeW1hbnRlYy5j
# b20wNwYIKwYBBQUHMAKGK2h0dHA6Ly90cy1haWEud3Muc3ltYW50ZWMuY29tL3Rz
# cy1jYS1nMi5jZXIwPAYDVR0fBDUwMzAxoC+gLYYraHR0cDovL3RzLWNybC53cy5z
# eW1hbnRlYy5jb20vdHNzLWNhLWcyLmNybDAoBgNVHREEITAfpB0wGzEZMBcGA1UE
# AxMQVGltZVN0YW1wLTIwNDgtMjAdBgNVHQ4EFgQURsZpow5KFB7VTNpSYxc/Xja8
# DeYwHwYDVR0jBBgwFoAUX5r1blzMzHSa1N197z/b7EyALt0wDQYJKoZIhvcNAQEF
# BQADggEBAHg7tJEqAEzwj2IwN3ijhCcHbxiy3iXcoNSUA6qGTiWfmkADHN3O43nL
# IWgG2rYytG2/9CwmYzPkSWRtDebDZw73BaQ1bHyJFsbpst+y6d0gxnEPzZV03LZc
# 3r03H0N45ni1zSgEIKOq8UvEiCmRDoDREfzdXHZuT14ORUZBbg2w6jiasTraCXEQ
# /Bx5tIB7rGn0/Zy2DBYr8X9bCT2bW+IWyhOBbQAuOA2oKY8s4bL0WqkBrxWcLC9J
# G9siu8P+eJRRw4axgohd8D20UaF5Mysue7ncIAkTcetqGVvP6KUwVyyJST+5z3/J
# vz4iaGNTmr1pdKzFHTx/kuDDvBzYBHUwggUpMIIEEaADAgECAgx1hq0Cs+a5vPW1
# iWcwDQYJKoZIhvcNAQELBQAwWjELMAkGA1UEBhMCQkUxGTAXBgNVBAoTEEdsb2Jh
# bFNpZ24gbnYtc2ExMDAuBgNVBAMTJ0dsb2JhbFNpZ24gQ29kZVNpZ25pbmcgQ0Eg
# LSBTSEEyNTYgLSBHMzAeFw0xNjEwMjgxMzUyMjVaFw0yMDAxMjMxMjIyMDhaMIGi
# MQswCQYDVQQGEwJERTEMMAoGA1UECBMDTlJXMREwDwYDVQQHEwhNdWVuc3RlcjEW
# MBQGA1UEChMNUFJPVklTSU8gR21iSDEWMBQGA1UECxMNUFJPVklTSU8gR21iSDEW
# MBQGA1UEAxMNUFJPVklTSU8gR21iSDEqMCgGCSqGSIb3DQEJARYbY29udGFjdC1l
# dXJvcGVAcHJvdmlzaW8uY29tMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEAq6jeVAo7UrorWLClM4eoW4WdWmHFOSWNWvdZrqNvPqbJuPw4OwsbB9wJ3vPC
# p9cfrnBaI3qdsfv3/u0JGjAhdpiaxgLz+69cZnZIZzjXn6Oub7FCPdhnAaaf8iOu
# ju5PFmqPBNRlO9Wn41HfPtW2n6y48qLFN27PuLcpKyjJ2uyu5VbCYuUPHPmVeIQ8
# GSoEOiundnJBN8SrY22H/PpEOcr+do92gu1V8vhwgbcii4iR5EfpQ9xvGw1fRrZP
# N3vUoyKEtGE6v2SuBz800brSd81OrBTpyWGNJsRFO7Ymnd29aPo8Ncs6zrn76/+4
# mv7aFoSiTY2clHrOvZJZgcRiCwIDAQABo4IBpDCCAaAwDgYDVR0PAQH/BAQDAgeA
# MIGUBggrBgEFBQcBAQSBhzCBhDBIBggrBgEFBQcwAoY8aHR0cDovL3NlY3VyZS5n
# bG9iYWxzaWduLmNvbS9jYWNlcnQvZ3Njb2Rlc2lnbnNoYTJnM29jc3AuY3J0MDgG
# CCsGAQUFBzABhixodHRwOi8vb2NzcDIuZ2xvYmFsc2lnbi5jb20vZ3Njb2Rlc2ln
# bnNoYTJnMzBWBgNVHSAETzBNMEEGCSsGAQQBoDIBMjA0MDIGCCsGAQUFBwIBFiZo
# dHRwczovL3d3dy5nbG9iYWxzaWduLmNvbS9yZXBvc2l0b3J5LzAIBgZngQwBBAEw
# CQYDVR0TBAIwADA/BgNVHR8EODA2MDSgMqAwhi5odHRwOi8vY3JsLmdsb2JhbHNp
# Z24uY29tL2dzY29kZXNpZ25zaGEyZzMuY3JsMBMGA1UdJQQMMAoGCCsGAQUFBwMD
# MB0GA1UdDgQWBBQg96pSCVfF8WIQuBHKI3hfDRWAsTAfBgNVHSMEGDAWgBQPOues
# lJF0LZYCc4OtnC5JPxmqVDANBgkqhkiG9w0BAQsFAAOCAQEAPcq4PM83C38Qof4w
# lPlN9Wuz6wmrPC81s5Of8ag1eFDYsrFZqFISTE4glvYgU5z97PNSfLfX/GaM+Rlo
# xtsDRbU5z5ZtTB9HJ1srzdXUBSUYGKBEFVuIE/MtvZrZ6pY8YEND9IoOe3K9szui
# udQR71UugCkA8OkFXuyQO9myx2nz66bA6k6QXeVif1PMwonmvGRhcA55bV+2Q4qR
# 1RjNIpcbEeg7CJqLXvFlACORHnY1daO3KrySrLpr1PAbhYEwzLsPBofItwBgZA8C
# skxAjFTYsKVe5xR6Wvgw2cFYppFFHM73ctB+LjTby3CouUMoCSiIfta57TiGB95z
# 6MsK/TGCBBowggQWAgEBMGowWjELMAkGA1UEBhMCQkUxGTAXBgNVBAoTEEdsb2Jh
# bFNpZ24gbnYtc2ExMDAuBgNVBAMTJ0dsb2JhbFNpZ24gQ29kZVNpZ25pbmcgQ0Eg
# LSBTSEEyNTYgLSBHMwIMdYatArPmubz1tYlnMAkGBSsOAwIaBQCgeDAYBgorBgEE
# AYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwG
# CisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBSxL1nc
# IXYE+1rq31wGGYcAC3kZODANBgkqhkiG9w0BAQEFAASCAQBPjZ+7tnKk1fEcfEca
# vwOU7iLzMS5lMrugJqKWXdaZIVehK5KrbwEk4OSfslXlCimYwzhBHEjMYLx4Escw
# SdVcb2y4gc/pq+rpon7MPcA0oQmpu/2QSzE7Qx5n5QcZoPqn4dDf+6aw3Rm07o2h
# A/AwaNyXa5UTb2JwmhKTdkrXZgta1IOZ80OiC6gq+gzTq6+cr8AQ75evUzhtzYpt
# VS6BOrd/4WpiRLgMJUah7zflImc9Km8p8W4Roja06hOHhwm0zY8CAqzho0euvzbx
# OBn4bbL4Senj5wR/8J8yMCISKjG0yN8Fxwtx5zLIPwwiCbJnjIX1W6rl7KO9WsJL
# 1xhOoYICCzCCAgcGCSqGSIb3DQEJBjGCAfgwggH0AgEBMHIwXjELMAkGA1UEBhMC
# VVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTAwLgYDVQQDEydTeW1h
# bnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIENBIC0gRzICEA7P9DjI/r81bgTY
# apgbGlAwCQYFKw4DAhoFAKBdMBgGCSqGSIb3DQEJAzELBgkqhkiG9w0BBwEwHAYJ
# KoZIhvcNAQkFMQ8XDTE5MDkzMDEwMTkzOFowIwYJKoZIhvcNAQkEMRYEFEu4FYkP
# NGSwafjPbHfaUqCD6bAzMA0GCSqGSIb3DQEBAQUABIIBACpEMPWigWTMwBxEQi2z
# qkp/Uaecz0d1SGMh1TJcl8zyEhumrP+yya0lX9weUKWcODTXdFz2mYcdqXKg3ty/
# /lfPxMmBAzc2uXf19ix+TUQglFlCmQzOrHlFpC3VpgafzVe263SzG8KseMUWZAqi
# xCEdflSrg3x0moztyq+eqoN18u+mHOWDwpLK6p9DLVC4o7PMVJHcv2mUwnpv5Y2d
# x3qOHQe94xsVZC6trPzMiVboKU8YIkkDtd6hFqMntzEMCQgKB9aMLcJk0veJQbib
# JpKR1sjwfxKUrESW80LsRU+dDQ6S6zBbErjhuX7LH7T97JJAYC4akUUOwJF546jc
# Rwg=
# SIG # End signature block
